-----------------------------------------------------------------------------------------------------------------------------------------
-- Entity Name:  EmifCoreGenFifo
-- File Name:    EmifCoreGenFifo.vhd
-- File Path:    E:\Projects\EmifTi\Design\Vhdl\CoregenFifo\Vhdl\
-- Project :	Connect a TI C64x DSP EMIF interface to a Xilinx FPGA
--
-- Purpose: 	Fifo interface.
--				Fifo is generated in Coregen as a 32-bit x 511 deep asynchronous fifo.
--				Two of these FIFOs are placed together, one in the EMIF read path and
--				one in the EMIF write pad. Fo rthe FPGA logic this looks oposite.
--
-- For implementation of the FIFO, please set the "Macro search path" to the correct
-- Location. It is the location where the EDIF file of the coregen generated module 
-- resides.
--
-- Limitations: none
--
-- Authors: Marc Defossez
--
-- Tools: 
--
--  Contact:    e-mail  hotline@xilinx.com        phone   + 1 800 255 7778
--
--  Disclaimer:
--              LIMITED WARRANTY AND DISCLAMER.
--              These designs are provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied, statutory or otherwise, and
--              Xilinx specifically disclaims any implied warranties of merchantability, non-infringement,
--              or fitness for a particular purpose. Xilinx does not warrant that the functions contained
--              in these designs will meet your requirements, or that the operation of these designs will
--              be uninterrupted or error free, or that defects in the designs will be corrected.
--              Furthermore, Xilinx does not warrant or make any representations regarding use or the
--              results of the use of the designs in terms of correctness, accuracy, reliability, or otherwise.
--
--              LIMITATION OF LIABILITY.
--              In no event will Xilinx or its licensors be liable for any loss of data, lost profits, cost or
--              procurement of substitute goods or services, or for any special, incidental, consequential,
--              or indirect damages arising from the use or operation of the designs or accompanying
--              documentation, however caused and on any theory of liability. This limitation will apply
--              even if Xilinx has been advised of the possibility of such damage. This limitation shall
--              apply not-withstanding the failure of the essential purpose of any limited remedies herein.
--
--              Copyright  2002 Xilinx, Inc.
--              All rights reserved
--
-- Revision History:       Created:       19/03/04
--                         Last opened:   Friday,Mar 19 04
--
----------------------------------------------------------------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_UNSIGNED.all;
-- Simulation libraries
	-- synopsys translate_off
	library UNISIM;
	use UNISIM.all;
	-- synopsys translate_on
-- end simulation libraries

-- Libraries for Synplicity
-- Neede for componnets as LUT, SRL which get an INIT parameter.
-- When this library is not mentionned Synplify_Pro fails during synthesis of
-- the hardware with an error about the INIT value.
 library virtex2;
 use virtex2.components.all;
-- This library is ignored by Mentor Precision.
-- Only a warning is generated.
-- End libraries for Synplicity

entity EmifCoreGenFifo is
    port (
        EmifRst		: in std_logic;
        EmifAreN	: in std_logic;
        EmifAweN	: in std_logic;
        EmifAoeN	: in std_logic;
        EmifCeN		: in std_logic;
        EmifEd		: inout std_logic_vector (31 downto 0);
        EmifIntX	: out std_logic;
        EmifIntY	: out std_logic;
        EmifIntZ	: out std_logic;
        
        SysDataIn		: in std_logic_vector (31 downto 0);
        SysDataOut		: out std_logic_vector (31 downto 0);
        SysClk          : in std_logic;
        SysRst          : in std_logic;
        SysRdEna        : in std_logic;
        SysWrEna        : in std_logic;
        SysRdCnt      	: out std_logic_vector (3 downto 0);
        SysAlmostFull 	: out std_logic;
        SysFull       	: out std_logic;
        SysWrCnt      	: out std_logic_vector (3 downto 0);
        SysAlmostEmpty	: out std_logic;
        SysEmpty      	: out std_logic
    );
end  EmifCoreGenFifo ;

architecture EmifCoreGenFifo_arch of EmifCoreGenFifo  is
----------------------------------------------------------------------------------------------------------------------------------------
-- Component Instantiation
----------------------------------------------------------------------------------------------------------------------------------------
-- For implementation of the FIFO, please set the "Macro search path" to the correct
-- Location. It is the location where the EDIF file of the coregen generated module 
-- resides.
component fifo_511x32
	port (
	din: 			IN std_logic_VECTOR(31 downto 0);
	wr_en: 			IN std_logic;
	wr_clk: 		IN std_logic;
	rd_en: 			IN std_logic;
	rd_clk: 		IN std_logic;
	ainit: 			IN std_logic;
	dout: 			OUT std_logic_VECTOR(31 downto 0);
	full: 			OUT std_logic;
	empty: 			OUT std_logic;
	almost_full: 	OUT std_logic;
	almost_empty: 	OUT std_logic;
	wr_count: 		OUT std_logic_VECTOR(3 downto 0);
	rd_count: 		OUT std_logic_VECTOR(3 downto 0)
	);
end component;

component IBUF_LVCMOS33
	port (
		I : in std_logic;
		O : out std_logic
	);
end component;
component OBUF_LVCMOS33_F_16
	port (
		I : in std_logic;
		O : out std_logic
	);
end component;
component IOBUF_LVCMOS33_F_24
	port (
		T : in std_logic;
		I : in std_logic;
		O : out std_logic;
		IO : inout std_logic
	);
end component;
----------------------------------------------------------------------------------------------------------------------------------------
-- Signal Declarations
----------------------------------------------------------------------------------------------------------------------------------------
signal IntEmifDataIn 	: std_logic_vector (31 downto 0);
signal IntEmifDataOut	: std_logic_vector (31 downto 0);
signal IntEmifAoeN   	: std_logic;
signal IntEmifCeN    	: std_logic;
signal IntEmifAweN   	: std_logic;
signal IntEmifAreN   	: std_logic;
signal IntEmifIntX   	: std_logic;
signal IntEmifIntY   	: std_logic;
signal IntEmifIntZ   	: std_logic;
signal IntEmifRst   	: std_logic;

signal IntWrEn			: std_logic;
signal IntRdEn			: std_logic;
signal IntWrClk			: std_logic;
signal IntRdClk			: std_logic;
signal IntDataIn	 	: std_logic_vector (31 downto 0);
signal IntDataOut		: std_logic_vector (31 downto 0);
signal IntWrWrCnt		: std_logic_vector (3 downto 0);
signal IntWrEmpty       : std_logic;
signal IntWrAlmostEmpty	: std_logic;
signal IntRdRdCnt       : std_logic_vector (3 downto 0);
signal IntRdAlmostFull  : std_logic;
signal IntRdFull        : std_logic;

signal Low : std_logic;

signal IntSysDataIn		: std_logic_vector (31 downto 0);
signal IntSysDataOut		: std_logic_vector (31 downto 0);
signal IntSysClk          	: std_logic;
signal IntSysRst          	: std_logic;
signal IntSysRdEna        	: std_logic;
signal IntSysWrEna        	: std_logic;
signal IntSysRdCnt      	: std_logic_vector (3 downto 0);
signal IntSysAlmostFull 	: std_logic;
signal IntSysFull       	: std_logic;
signal IntSysWrCnt      	: std_logic_vector (3 downto 0);
signal IntSysAlmostEmpty	: std_logic;
signal IntSysEmpty      	: std_logic;
----------------------------------------------------------------------------------------------------------------------------------------
--
begin

Low <= '0';
-----------------------------------------------------------
-- Io buffers
-----------------------------------------------------------
RstBuf : IBUF_LVCMOS33
	port map (
		I => EmifRst,
		O => IntEmifRst
	);
DataGen : for d in 31 downto 0 generate
	BufIo : IOBUF_LVCMOS33_F_24
	port map (
		T  => Low,
		I  => IntEmifDataOut(d),
		O  => IntEmifDataIn(d),
		IO => EmifEd(d)
	);
end generate;
AoeBuf : IBUF_LVCMOS33
	port map (
		I => EmifAoeN,
		O => IntEmifAoeN
	);
CeBuf : IBUF_LVCMOS33
	port map (
		I => EmifCeN,
		O => IntEmifCeN
	);
AweBuf : IBUF_LVCMOS33
	port map (
		I => EmifAweN,
		O => INtEmifAweN
	);
AreBuf : IBUF_LVCMOS33
	port map (
		I => EmifAreN,
		O => IntEmifAreN
	);

IntxBuf : OBUF_LVCMOS33_F_16
	port map (
		I => IntEmifIntX,
		O => EmifIntX
	);
IntyBuf : OBUF_LVCMOS33_F_16
	port map (
		I => IntEmifIntY,
		O => EmifIntY
	);
IntzBuf : OBUF_LVCMOS33_F_16
	port map (
		I => IntEmifIntZ,
		O => EmifIntZ
	);
-----------------------------------------------------------
-- Gates
-----------------------------------------------------------
IntWrEn <= not IntEmifAoeN and IntEmifCeN;
IntRdEn <= not IntEmifAoeN and not IntEmifCeN;
IntWrClk <= not IntEmifAweN;
IntRdClk <= not IntEmifAreN;

DataDirLogic : for n in 31 downto 0 generate
	IntEmifDataOut(n) <= IntDataOut(n) and IntWrEn;
	IntDataIn(n) <= IntEmifDataIn(n) and IntRdEn;
end generate;
-----------------------------------------------------------
-- Fifo components
-----------------------------------------------------------
TheWriteFifo : fifo_511x32
		port map (
			din 			=> IntDataIn,
			wr_en 			=> IntWrEn,
			wr_clk 			=> IntWrClk,
			rd_en	 		=> SysRdEna,
			rd_clk 			=> SysClk,
			ainit 			=> IntEmifRst,
			dout 			=> IntSysDataOut,
			full 			=> IntSysFull,
			empty 			=> IntWrEmpty,
			almost_full 	=> IntSysAlmostFull,
			almost_empty	=> IntWrAlmostEmpty,
			wr_count 		=> IntWrWrCnt,
			rd_count 		=> IntSysRdCnt
			);

TheReadFifo : fifo_511x32
		port map (
			din 			=> IntSysDataIn,
			wr_en 			=> SysWrEna,
			wr_clk 			=> SysClk,
			rd_en	 		=> IntRdEn,
			rd_clk 			=> IntRdClk,
			ainit 			=> SysRst,
			dout 			=> IntDataOut,
			full 			=> IntRdFull,
			empty 			=> IntSysEmpty,
			almost_full 	=> IntRdAlmostFull,
			almost_empty	=> IntSysAlmostEmpty,
			wr_count 		=> IntSysWrCnt,
			rd_count 		=> IntRdRdCnt
			);
-----------------------------------------------------------
-- Fifo System back side
-----------------------------------------------------------
SysData : process (SysClk, SysRst)
begin
	if (SysRst = '1') then
		SysDataOut <= (others => '0');
		IntSysDataIn <= (others => '0');
	elsif (SysClk'event and SysClk = '1') then
		if (SysRdEna = '1') then
			SysDataOut <= IntSysDataOut;
		end if;
		if (SysWrEna = '1') then
			IntSysDataIn <= SysDataIn;
		end if;
	end if;
end process;

SysFlags : process (SysClk, SysRst)
begin
	if (SysRSt = '1') then
		SysRdCnt       <= (others => '0');
        SysAlmostFull  <= '0';
        SysFull        <= '0';
        SysWrCnt       <= (others => '0');
        SysAlmostEmpty <= '0';
        SysEmpty       <= '0';
	elsif (SysClk'event and SysClk = '1') then
		if (SysRdEna = '1') then
			SysRdCnt       <= IntSysRdCnt;
        	SysAlmostFull  <= IntSysAlmostFull;
        	SysFull        <= IntSysFull;
        end if;
        if (SysWrEna = '1') then
        	SysWrCnt       <= IntSysWrCnt;
        	SysAlmostEmpty <= IntSysAlmostEmpty;
        	SysEmpty       <= IntSysEmpty;
        end if;
    end if;
end process;
-----------------------------------------------------------
end EmifCoreGenFifo_arch ;
