-----------------------------------------------------------------------------------------------------------------------------------------
-- Entity Name:  LargeSelfAddrFifo
-- File Name:    LargeSelfAddrFifo.vhd
-- File Path:    E:\Projects\PhilipsSemicon\Design\Vhdl\LargeRxFifo\
-- Project :
--
-- Purpose: 
--
-- Limitations: none
--
-- Authors: Marc Defossez
--
-- Tools: 
--
--  Contact:    e-mail  hotline@xilinx.com        phone   + 1 800 255 7778
--
--  Disclaimer:
--              LIMITED WARRANTY AND DISCLAMER.
--              These designs are provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied, statutory or otherwise, and
--              Xilinx specifically disclaims any implied warranties of merchantability, non-infringement,
--              or fitness for a particular purpose. Xilinx does not warrant that the functions contained
--              in these designs will meet your requirements, or that the operation of these designs will
--              be uninterrupted or error free, or that defects in the designs will be corrected.
--              Furthermore, Xilinx does not warrant or make any representations regarding use or the
--              results of the use of the designs in terms of correctness, accuracy, reliability, or otherwise.
--
--              LIMITATION OF LIABILITY.
--              In no event will Xilinx or its licensors be liable for any loss of data, lost profits, cost or
--              procurement of substitute goods or services, or for any special, incidental, consequential,
--              or indirect damages arising from the use or operation of the designs or accompanying
--              documentation, however caused and on any theory of liability. This limitation will apply
--              even if Xilinx has been advised of the possibility of such damage. This limitation shall
--              apply not-withstanding the failure of the essential purpose of any limited remedies herein.
--
--              Copyright  2002 Xilinx, Inc.
--              All rights reserved
--
-- Revision History:       Created:       11/03/04
--                         Last opened:   Thursday,Mar 11 04
--
----------------------------------------------------------------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_UNSIGNED.all;
-- Simulation libraries
	-- synopsys translate_off
	library UNISIM;
	use UNISIM.all;
	-- synopsys translate_on
-- end simulation libraries

-- Libraries for Synplicity
-- Neede for componnets as LUT, SRL which get an INIT parameter.
-- When this library is not mentionned Synplify_Pro fails during synthesis of
-- the hardware with an error about the INIT value.
-- library virtex2;
-- use virtex2.components.all;
-- This library is ignored by Mentor Precision.
-- Only a warning is generated.
-- End libraries for Synplicity

entity LargeSelfAddrFifo is
    port (
        ClkPortA	: in std_logic;
        RstPortA	: in std_logic;
        EnaPortA	: in std_logic;
        WrePortA	: in std_logic;
        DatInPortA	: in std_logic_vector (31 downto 0);
        
        FlagE		: out std_logic;	-- Empty flag. Fifo between empty and 1/4.
        FlagEh		: out std_logic;	-- Empty-Half flag. Fifo between 1/4 and half.
        FlagHf		: out std_logic;	-- Half-Full flag. Fifo between half and 3/4.
        FlagF		: out std_logic;	-- Full flag. Fifo between 3/4 and full.
                
        ClkPortB	: in std_logic;
        RstPortB	: in std_logic;
        EnaPortB	: in std_logic;
        DatOutportB	: out std_logic_vector (31 downto 0)
    );
end  LargeSelfAddrFifo ;

architecture LargeSelfAddrFifo_arch of LargeSelfAddrFifo  is
----------------------------------------------------------------------------------------------------------------------------------------
-- Component Instantiation
----------------------------------------------------------------------------------------------------------------------------------------
component RAMB16_S36_S36
	port (
		WEA		: in std_logic;
		ENA		: in std_logic;
		SSRA	: in std_logic;
		CLKA	: in std_logic;
		ADDRA	: in std_logic_vector (8 downto 0);
		DIA		: in std_logic_vector (31 downto 0);
		DIPA	: in std_logic_vector (3 downto 0);
		DOPA	: out std_logic_vector (3 downto 0);
		DOA		: out std_logic_vector (31 downto 0);
		
		WEB		: in std_logic;
		ENB		: in std_logic;
		SSRB	: in std_logic;
		CLKB	: in std_logic;
		ADDRB	: in std_logic_vector (8 downto 0); 
		DIB		: in std_logic_vector (31 downto 0);
		DIPB	: in std_logic_vector (3 downto 0); 
		DOPB	: out std_logic_vector (3 downto 0); 
		DOB		: out std_logic_vector (31 downto 0)
	);
end component;

component GrayCnt4b
    port (
    	Clk			: in std_logic;
        Rst			: in std_logic;
        Ena			: in std_logic;
        CmpTc4b		: out std_logic;
        FfTc4b		: out std_logic;
        IncOut4b	: out std_logic_vector (3 downto 0);
        CntOut4b	: out std_logic_vector (3 downto 0)
    );
end component;

component LUT4
	generic (
		INIT	: bit_vector (15 downto 0)
		);
	port (
		I0		: in std_logic;
		I1		: in std_logic;
		I2		: in std_logic;
		I3		: in std_logic;
		O		: out std_logic
		);
end component;

component FDCE
	port (
		D		: in std_logic;
		CE		: in std_logic;
		C		: in std_logic;
		CLR		: in std_logic;
		Q		: out std_logic
	);
end component;
----------------------------------------------------------------------------------------------------------------------------------------
-- Signal Declarations
----------------------------------------------------------------------------------------------------------------------------------------
signal High			: std_logic;
signal Low			: std_logic;
signal Int4bitLow	: std_logic_vector (3 downto 0);
signal Int32bitLow	: std_logic_vector (31 downto 0);

signal IntEmpty		: std_logic;
signal IntFull		: std_logic;
signal IntHalf		: std_logic;

signal IntAddrPortB			: std_logic_vector (8 downto 0);
signal IntWrLsbTc			: std_logic;
signal IntWrMsbEna			: std_logic;
signal IntWrLsbBitsCnt		: std_logic_vector (3 downto 0);
signal IntWrMsbBit			: std_logic;
signal IntWrMsbBitInv		: std_logic;
signal IntWrCntOut			: std_logic_vector (4 downto 0);

signal IntNxtWriteAddr			: std_Logic_vector (3 downto 0);
signal IntNxtWriteAddrPlusOne	: std_Logic_vector (3 downto 0);
signal IntNxtWriteAddrTc		: std_Logic;
signal IntWriteAddr				: std_Logic_vector (8 downto 0);

signal IntRdLsbTc			: std_logic;
signal IntRdMsbEna			: std_logic;
signal IntRdLsbBitsCnt		: std_logic_vector (3 downto 0);
signal IntRdMsbBit			: std_logic;
signal IntRdMsbBitInv		: std_logic;
signal IntRdCntOut			: std_logic_vector (4 downto 0);

signal IntNxtReadAddr		: std_Logic_vector (3 downto 0);
signal IntNxtReadAddrTc		: std_logic;

type InitVal is array (3 downto 0) of bit_vector (15 downto 0);
constant AddrIncLutVal : InitVal := (
	X"FE10",	-- Address tree
	X"B0F4",
	X"E44E",
	X"3CC3"		-- Address zero
);
constant AddrWrTcLutVal : bit_vector (15 downto 0) := X"0100";
constant AddrRdTcLutVal : bit_vector (15 downto 0) := X"0100";

-- Synplicity attributes
attribute SYN_HIER : string;
	attribute SYN_HIER of LargeSelfAddrFifo_arch : architecture is "hard";
-- Xilinx attributes
----------------------------------------------------------------------------------------------------------------------------------------
--
begin
--
------------------------------------------------------------------------------------------
-- Write Port A
------------------------------------------------------------------------------------------
-- Gray counter / incrementer for lower 4 address bits.
-- Incrementer existing as a set of LUT's.
-- These feed the Parity Data inputs of the block SelectRAM.
-- At the output of the RAM thee bits are used as lower address bits.
WrAddrLut : for n in 0 to 3 generate
	IncRom : LUT4
		generic map (
			INIT => AddrIncLutVal(n)
		)
		port map (
			I0 	=> IntNxtWriteAddr(0),
			I1 	=> IntNxtWriteAddr(1),
			I2 	=> IntNxtWriteAddr(2),
			I3 	=> IntNxtWriteAddr(3),
			O 	=> IntNxtWriteAddrPlusOne(n)
		);
end generate;
WrAddrTcLut : LUT4
	generic map (
		INIT => AddrWrTcLutVal
	)
	port map (
		I0 	=> IntNxtWriteAddr(0),
		I1 	=> IntNxtWriteAddr(1),
		I2 	=> IntNxtWriteAddr(2),
		I3 	=> IntNxtWriteAddr(3),
		O 	=> IntNxtWriteAddrTc
	);
-- Gray counter / incrementer for upper 5 address bits.
-- These bits are added to the four lower address bits forming the 9-bit address.
LowWrCnt : GrayCnt4b
   	port map (
    	Clk			=> ClkPortA,
   	    Rst			=> RstPortA,
       	Ena			=> IntNxtWriteAddrTc,
        CmpTc4b		=> IntWrLsbTc,
   	    FfTc4b		=> open,
   	    IncOut4b	=> open,
       	CntOut4b	=> IntWrLsbBitsCnt
   	);
IntWrMsbEna <= IntWrLsbTc and IntNxtWriteAddrTc;
HigWrCnt : FDCE
	port map (
		D		=> IntWrMsbBitInv,
		CE		=> IntWrMsbEna,
		C		=> ClkPortA,
		CLR		=> RstPortA,
		Q		=> IntWrMsbBit
	);
IntWrMsbBitInv <= not IntWrMsbBit;
IntWrCntOut <= IntWrMsbBit & IntWrLsbBitsCnt;
------------------------------------------------------------------------------------------
-- Memory : Block SelectRAM
------------------------------------------------------------------------------------------
IntWriteAddr <= IntWrCntOut & IntNxtWriteAddr;

TheFifoRam : RAMB16_S36_S36
	port map (
		WEA		=> WrePortA,
		ENA		=> EnaPortA,
		SSRA	=> RstPortA,
		CLKA	=> ClkPOrtA,
		ADDRA	=> IntWriteAddr,
		DIA		=> DatInPortA,
		DIPA	=> IntNxtWriteAddrPlusOne,
		DOPA	=> IntNxtWriteAddr,
		DOA		=> open,

		WEB		=> Low,
		ENB		=> EnaPortB,
		SSRB	=> RstPortB,
		CLKB	=> ClkPortB,
		ADDRB	=> IntAddrPortB,
		DIB		=> Int32bitLow,
		DIPB	=> Int4BitLow,
		DOPB	=> IntNxtReadAddr,
		DOB		=> DatOutPortB
	);
Int4bitLow <=X"0";
Int32bitLow <=X"00000000";
IntAddrPortB <= IntRdCntOut & IntNxtReadAddr;

------------------------------------------------------------------------------------------
-- Read Port B
------------------------------------------------------------------------------------------
RdAddrTcLut : LUT4
	generic map (
		INIT => AddrRdTcLutVal
	)
	port map (
		I0 	=> IntNxtReadAddr(0),	-- WHen nible n-1 generate a TC.
		I1 	=> IntNxtReadAddr(1),
		I2 	=> IntNxtReadAddr(2),
		I3 	=> IntNxtReadAddr(3),
		O 	=> IntNxtReadAddrTc
	);
-- Gray counter / incrementer for upper 5 address bits.
-- These bits are added to the four lower address bits forming the 9-bit address.
LowRdCnt : GrayCnt4b
   	port map (
    	Clk			=> ClkPortB,
   	    Rst			=> RstPortB,
       	Ena			=> IntNxtReadAddrTc,
        CmpTc4b		=> IntRdLsbTc,
   	    FfTc4b		=> open,
   	    IncOut4b	=> open,
       	CntOut4b	=> IntRdLsbBitsCnt
   	);
IntRdMsbEna <= IntRdLsbTc and IntNxtReadAddrTc;
HigRdCnt : FDCE
	port map (
		D		=> IntRdMsbBitInv,
		CE		=> IntRdMsbEna,
		C		=> ClkPortB,
		CLR		=> RstPortB,
		Q		=> IntRdMsbBit
	);
IntRdMsbBitInv <= not IntRdMsbBit;
IntRdCntOut <= IntRdMsbBit & IntRdLsbBitsCnt;
------------------------------------------------------------------------------------------
-- Flags
------------------------------------------------------------------------------------------
-- The flags decoded give an indication of the status of the fifo.
-- When more precise flags are needed, for burst or packet based designs,
-- following signals can be used.
-- 	IntNxtWriteAddrTc	Terminal count of the lower 4-bits (3:0) of the write address.
--	IntWrLsbTc			Terminal count of the four next bits (7:4) of the write address
--						This signal will stay as long as the terminal count value is valid.
--	IntWrMsbEna			One clock pulse wide terminal count puls of address bits (7:4)
--	IntWrCntOut			All address bits of the write fifo port (BRAM port A)
-- idem for the read port side of the fifo.
--	IntNxtReadAddrTc
--	IntRdLsbTc
--	IntRdMsbEna
--	IntRdCntOut
--
-- Empty flag. Fifo between empty and 1/4.
FlagE
-- Empty-Half flag. Fifo between 1/4 and half.
FlagEh
-- Half-Full flag. Fifo between half and 3/4.
FlagHf
-- Full flag. Fifo between 3/4 and full.
FlagF
------------------------------------------------------------------------------------------
end  LargeSelfAddrFifo_arch ;