-----------------------------------------------------------------------------------------------------------------------------------------
-- Entity Name:  AltEmifInt_TestBench
-- File Name:    AltEmifInt_TestBench.vhd
-- File Path:    E:\Projects\AltEmifInt\Vhdl\
-- Project :		Provide a FPGA to TI C64x DSP EMIF interface
--
-- TESTBENCH, DON'T USE THIS FILE FOR COMPILATION !
-- Purpose: 
--
-- Limitations: none
--
-- Authors: Marc Defossez
--
-- Tools: 
--
--  Contact:    e-mail  hotline@xilinx.com        phone   + 1 800 255 7778
--
--  Disclaimer:
--              LIMITED WARRANTY AND DISCLAMER.
--              These designs are provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied, statutory or otherwise, and
--              Xilinx specifically disclaims any implied warranties of merchantability, non-infringement,
--              or fitness for a particular purpose. Xilinx does not warrant that the functions contained
--              in these designs will meet your requirements, or that the operation of these designs will
--              be uninterrupted or error free, or that defects in the designs will be corrected.
--              Furthermore, Xilinx does not warrant or make any representations regarding use or the
--              results of the use of the designs in terms of correctness, accuracy, reliability, or otherwise.
--
--              LIMITATION OF LIABILITY.
--              In no event will Xilinx or its licensors be liable for any loss of data, lost profits, cost or
--              procurement of substitute goods or services, or for any special, incidental, consequential,
--              or indirect damages arising from the use or operation of the designs or accompanying
--              documentation, however caused and on any theory of liability. This limitation will apply
--              even if Xilinx has been advised of the possibility of such damage. This limitation shall
--              apply not-withstanding the failure of the essential purpose of any limited remedies herein.
--
--              Copyright  2002 Xilinx, Inc.
--              All rights reserved
--
-- Revision History:       Created:       08/03/04
--                         Last opened:   Monday,Mar 08 04
--                                        12:57:39
--
----------------------------------------------------------------------------------------------------------------------------------------
--
library IEEE ;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_UNSIGNED.all;
use STD.textio.all;

entity AltEmifInt_TestBench is
-- Declarations
end AltEmifInt_TestBench ;

architecture AltEmifInt_struct of AltEmifInt_TestBench is

-- Architecture declarations

-- Internal signal declarations
-- EMIF port / Port A
constant NumberOfEmifDataBits : integer := 32;
constant NumberOfEmifAddrBits : integer := 9;
-- System Port / Port B
constant NumberOfSysDataBits : integer := 32;
constant NumberOfSysAddrBits : integer := 9;
-- Block SelectRAM
constant NumberOfBlockRamBits : integer := 32;
--
-- EMIF side
signal SimEmifClk		: std_logic;
signal SimEmifRst		: std_logic;
signal SimEmifAddr		: std_logic_vector (NumberOfEmifAddrBits-1 downto 0);
signal SimEmifData		: std_logic_vector (NumberOfEmifDataBits-1 downto 0);
signal SimEmifSoe		: std_logic;
signal SimEmifCe		: std_logic;
signal SimEmifSwe		: std_logic;
signal SimEmifSads		: std_logic;
signal SimEmifBe		: std_logic_vector (((NumberOfEmifDataBits/8)-1) downto 0);
-- Internal or system side
signal SimSysClk		: std_logic;
signal SimSysRst		: std_logic;
signal SimSysAddr		: std_logic_vector (NumberOfSysAddrBits-1 downto 0);     -- Address can be different width.
signal SimSysData		: std_logic_vector (NumberOfSysDataBits-1 downto 0);  -- Data can be different width.
signal SimSysRdWr		: std_logic;
signal SimSysEna		: std_logic_vector (((NumberOfSysDataBits/NumberOfBlockRamBits)-1) downto 0);

-- Component Declarations
component AltEmifInt
	generic (
		-- EMIF port / Port A
		NumberOfDataBits 	: integer;
		NumberOfAddrBits 	: integer;
		-- System Port / Port B
		NumberOfSysDataBits : integer;
		NumberOfSysAddrBits : integer;
		-- Block SelectRAM
		UsedBlockRamWidth 	: integer
	);
    port (
        -- EMIF side
        EmifClk		: in std_logic;
        EmifRst		: in std_logic;
        EmifAddr	: in std_logic_vector (NumberOfAddrBits-1 downto 0);
        EmifData	: inout std_logic_vector (NumberOfDataBits-1 downto 0);
        EmifSoe		: in std_logic;
        EmifCe		: in std_logic;
        EmifSwe		: in std_logic;
        EmifSads	: in std_logic;
        EmifBe		: in std_logic_vector (((NumberOfDataBits/8)-1) downto 0);
        -- Internal or system side
        SysClk		: in std_logic;
        SysRst		: in std_logic;
        SysAddr		: in std_logic_vector (NumberOfSysAddrBits-1 downto 0);     -- Address can be different width.
        SysData		: Inout std_logic_vector (NumberOfSysDataBits-1 downto 0);  -- Data can be different width.
        SysRdWr		: in std_logic;
        SysEna		: in std_logic_vector (((NumberOfSysDataBits/UsedBlockRamWidth)-1) downto 0)
    );
end component;

component AltEmifInt_Tester
	generic (
		-- EMIF port / Port A
		NumberOfDataBits 	: integer;
		NumberOfAddrBits 	: integer;
		-- System Port / Port B
		NumberOfSysDataBits : integer;
		NumberOfSysAddrBits : integer;
		-- Block SelectRAM
		UsedBlockRamWidth 	: integer
	);
    port (
        -- EMIF side
        EmifClk		: out std_logic;
        EmifRst		: out std_logic;
        EmifAddr	: out std_logic_vector (NumberOfAddrBits-1 downto 0);
        EmifData	: inout std_logic_vector (NumberOfDataBits-1 downto 0);
        EmifSoe		: out std_logic;
        EmifCe		: out std_logic;
        EmifSwe		: out std_logic;
        EmifSads	: out std_logic;
        EmifBe		: out std_logic_vector (((NumberOfDataBits/8)-1) downto 0);
        -- Internal or system side
        SysClk		: out std_logic;
        SysRst		: out std_logic;
        SysAddr		: out std_logic_vector (NumberOfSysAddrBits-1 downto 0);     -- Address can be different width.
        SysData		: Inout std_logic_vector (NumberOfSysDataBits-1 downto 0);  -- Data can be different width.
        SysRdWr		: out std_logic;
        SysEna		: out std_logic_vector (((NumberOfSysDataBits/UsedBlockRamWidth)-1) downto 0)
	);
end component;

begin
-- port mappings.
I0 : AltEmifInt
	generic map (
		-- EMIF port / Port A
		NumberOfDataBits 	=> NumberOfEmifDataBits,
		NumberOfAddrBits 	=> NumberOfEmifAddrBits,
		-- System Port / Port B
		NumberOfSysDataBits => NumberOfSysDataBits,
		NumberOfSysAddrBits => NumberOfSysAddrBits,
		-- Block SelectRAM
		UsedBlockRamWidth 	=> NumberOfBlockRamBits
	)
    port map (
        EmifClk		=> SimEmifClk,
        EmifRst		=> SimEmifRst,
        EmifAddr	=> SimEmifAddr,
        EmifData	=> SimEmifData,
        EmifSoe		=> SimEmifSoe,
        EmifCe		=> SimEmifCe,
        EmifSwe		=> SimEmifSwe,
        EmifSads	=> SimEmifSads,
        EmifBe		=> SimEmifBe,
        SysClk		=> SimSysClk,
        SysRst		=> SimSysRst,
        SysAddr		=> SimSysAddr,
        SysData		=> SimSysData,
        SysRdWr		=> SimSysRdWr,
        SysEna		=> SimSysEna
        );
I1 : AltEmifInt_Tester
	generic map (
		-- EMIF port / Port A
		NumberOfDataBits 	=> NumberOfEmifDataBits,
		NumberOfAddrBits 	=> NumberOfEmifAddrBits,
		-- System Port / Port B
		NumberOfSysDataBits => NumberOfSysDataBits,
		NumberOfSysAddrBits => NumberOfSysAddrBits,
		-- Block SelectRAM
		UsedBlockRamWidth 	=> NumberOfBlockRamBits
	)
    port map (
        EmifClk		=> SimEmifClk,
        EmifRst		=> SimEmifRst,
        EmifAddr	=> SimEmifAddr,
        EmifData	=> SimEmifData,
        EmifSoe		=> SimEmifSoe,
        EmifCe		=> SimEmifCe,
        EmifSwe		=> SimEmifSwe,
        EmifSads	=> SimEmifSads,
        EmifBe		=> SimEmifBe,
        SysClk		=> SimSysClk,
        SysRst		=> SimSysRst,
        SysAddr		=> SimSysAddr,
        SysData		=> SimSysData,
        SysRdWr		=> SimSysRdWr,
        SysEna		=> SimSysEna
    );

end AltEmifInt_struct;
