-----------------------------------------------------------------------------------------------------------------------------------------
-- Entity Name:  AltEmifInt_Tester
-- File Name:    AltEmifInt_Tester.vhd
-- File Path:    E:\Projects\AltEmifInt\Vhdl\
-- Project :
--
-- TESTER, DON'T USE THIS FILE FOR COMPILATION !
-- Purpose: 
--
-- Limitations: none
--
-- Authors: Marc Defossez
--
-- Tools: 
--
--  Contact:    e-mail  hotline@xilinx.com        phone   + 1 800 255 7778
--
--  Disclaimer:
--              LIMITED WARRANTY AND DISCLAMER.
--              These designs are provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied, statutory or otherwise, and
--              Xilinx specifically disclaims any implied warranties of merchantability, non-infringement,
--              or fitness for a particular purpose. Xilinx does not warrant that the functions contained
--              in these designs will meet your requirements, or that the operation of these designs will
--              be uninterrupted or error free, or that defects in the designs will be corrected.
--              Furthermore, Xilinx does not warrant or make any representations regarding use or the
--              results of the use of the designs in terms of correctness, accuracy, reliability, or otherwise.
--
--              LIMITATION OF LIABILITY.
--              In no event will Xilinx or its licensors be liable for any loss of data, lost profits, cost or
--              procurement of substitute goods or services, or for any special, incidental, consequential,
--              or indirect damages arising from the use or operation of the designs or accompanying
--              documentation, however caused and on any theory of liability. This limitation will apply
--              even if Xilinx has been advised of the possibility of such damage. This limitation shall
--              apply not-withstanding the failure of the essential purpose of any limited remedies herein.
--
--              Copyright  2002 Xilinx, Inc.
--              All rights reserved
--
-- Revision History:       Created:       08/03/04
--                         Last opened:   Monday,Mar 08 04
--                                        13:14:34
--
----------------------------------------------------------------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_UNSIGNED.all;
use IEEE.std_logic_textio.all;
use std.textio.all;
use IEEE.std_logic_arith.all;

entity AltEmifInt_Tester is
	generic (
		-- EMIF port / Port A
		NumberOfDataBits 	: integer;
		NumberOfAddrBits 	: integer;
		-- System Port / Port B
		NumberOfSysDataBits : integer;
		NumberOfSysAddrBits : integer;
		-- Block SelectRAM
		UsedBlockRamWidth 	: integer
	);
    port (
        -- EMIF side
        EmifClk		: out std_logic;
        EmifRst		: out std_logic;
        EmifAddr	: out std_logic_vector (NumberOfAddrBits-1 downto 0);
        EmifData	: inout std_logic_vector (NumberOfDataBits-1 downto 0);
        EmifSoe		: out std_logic;
        EmifCe		: out std_logic;
        EmifSwe		: out std_logic;
        EmifSads	: out std_logic;
        EmifBe		: out std_logic_vector (((NumberOfDataBits/8)-1) downto 0);
        -- Internal or system side
        SysClk		: out std_logic;
        SysRst		: out std_logic;
        SysAddr		: out std_logic_vector (NumberOfSysAddrBits-1 downto 0);     -- Address can be different width.
        SysData		: Inout std_logic_vector (NumberOfSysDataBits-1 downto 0);  -- Data can be different width.
        SysRdWr		: out std_logic;
        SysEna		: out std_logic_vector (((NumberOfSysDataBits/UsedBlockRamWidth)-1) downto 0)
    );
end AltEmifInt_Tester;

architecture AltEmifInt_flow of AltEmifInt_Tester is
-- Functions
function str_to_stdlvec(inp: string) return std_logic_vector is
	variable temp: std_logic_vector(inp'range) := (others => 'X');
begin 
	for i in inp'range loop
		if (inp(i) = '1') then
			temp(i) := '1';
		elsif (inp(i) = '0') then
			temp(i) := '0'; 
		end if;
	end loop;
return temp;
end function str_to_stdlvec;

function stdlvec_to_str(inp: std_logic_vector) return string is
	variable temp: string(inp'left+1 downto 1) := (others => 'X');
begin
	for i in inp'reverse_range loop
		if (inp(i) = '1') then
			temp(i+1) := '1';
		elsif (inp(i) = '0') then
			temp(i+1) := '0'; 
		end if;
	end loop;
return temp;
end function stdlvec_to_str;

-- Architecture declarations
constant EmifClockPeriod : time := 5.5 ns;
constant SysClockPeriod : time := 10 ns;
signal IntEmifClk : std_logic;
signal IntSysClk : std_logic;

begin

MainProc : process
--file TextFile : text is in "AltEmifIntDat.vec"; -- OLD VHDL syntax, ModelSim gives a warning.
file TextFile : text;
variable LineLength : line;
variable DatIn : string(32 downto 1);
--
begin
	wait for 50 ns;

--
	FILE_OPEN (TextFile, "E:\Projects\EmifTi\Design\Simscripts\AltEmifIntDat.vec", READ_MODE);
	ReadCharacters : while not endfile (TextFile) loop
		readline (TextFile, LineLength);
		read (LineLength, DatIn);
		EmifData <= str_to_stdlvec(DatIn);
		assert false
		report "Read data in file"
		severity warning;
		wait for 2.5 ns;			-- Data apply time smaller than a clock cycle.
	end loop;
--
	assert false
	report "That's All FOlks !"
	severity warning;
    wait;
end process MainProc;


-- Architecture concurrent statements
-- generate the clock for port A
EmifClockGen : process
begin
   IntEmifClk <= '0';
   wait for EmifClockPeriod/2;
   IntEmifClk <= '1';
   wait for EmifClockPeriod/2;
end process;
EmifClk <= IntEmifClk;

-- generate the clock for port B
SysClockGen : process
begin
   IntSysClk <= '0';
   wait for SysClockPeriod/2;
   IntSysClk <= '1';
   wait for SysClockPeriod/2;
end process;
SysClk <= IntSysClk;

end AltEmifInt_flow;
