-----------------------------------------------------------------------------------------------------------------------------------------
-- Entity Name:  V4SmplFfsEmifC64x
-- File Name:    V4SmplFfsEmifC64x.vhd
-- File Path:    \Projects\EmifTi\EMIF_V4\Vhdl
-- Project :	Virtex-4 EMIF interafce 
--
-- Purpose: 	Connect a Virtex-4 device to a TI C64x DSP per EMIF interface.
--
-- Limitations: none
--
-- Authors: Marc Defossez
--
-- Tools: 
--
--  Contact:    e-mail  hotline@xilinx.com        phone   + 1 800 255 7778
--
--  Disclaimer:
--              LIMITED WARRANTY AND DISCLAIMER.
--              These designs are provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied, statutory or otherwise, and
--              Xilinx specifically disclaims any implied warranties of merchantability, non-infringement,
--              or fitness for a particular purpose. Xilinx does not warrant that the functions contained
--              in these designs will meet your requirements, or that the operation of these designs will
--              be uninterrupted or error free, or that defects in the designs will be corrected.
--              Furthermore, Xilinx does not warrant or make any representations regarding use or the
--              results of the use of the designs in terms of correctness, accuracy, reliability, or otherwise.
--
--              LIMITATION OF LIABILITY.
--              In no event will Xilinx or its licensors be liable for any loss of data, lost profits, cost or
--              procurement of substitute goods or services, or for any special, incidental, consequential,
--              or indirect damages arising from the use or operation of the designs or accompanying
--              documentation, however caused and on any theory of liability. This limitation will apply
--              even if Xilinx has been advised of the possibility of such damage. This limitation shall
--              apply not-withstanding the failure of the essential purpose of any limited remedies herein.
--
--              Copyright  2002 Xilinx, Inc.
--              All rights reserved
--
-- Revision History:		Created:		17/01/05
--							Last opened:	Monday,Jan 17 05
--							Revision 01.00:	Friday 18 Feb. 2005
----------------------------------------------------------------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_UNSIGNED.all;
-- Simulation libraries
	-- synopsys translate_off
	library UNISIM;
	use UNISIM.all;
	-- synopsys translate_on
-- end simulation libraries

-- Libraries for Synplicity
-- Needed for components as LUT, SRL which get an INIT parameter.
-- When this library is not mentioned Synplify_Pro fails during synthesis of
-- the hardware with an error about the INIT value.
-- library virtex2;
-- use virtex2.components.all;
-- This library is ignored by Mentor Precision.
-- Only a warning is generated.
-- End libraries for Synplicity

entity V4SmplFfsEmifC64x is
    port (
        Rst 		:	in std_logic;
        DspEclkOut	:	in std_logic;
        DspCEn		: 	in std_logic;
        DspSWEn		: 	in std_logic;
        DspSREn		: 	in std_logic;
        DspSOEn		: 	in std_logic;
        DspBEn		:	in std_logic_vector(3 downto 0);
        DspEA		:	in std_logic_vector(11 downto 0);
        DspED		:	inout std_logic_vector(31 downto 0);
        
        FpgaClk		:	in std_logic;
        SysEna		:	in std_logic;
        SysWre		:	in std_logic;
        SysDatIo	:	inout std_logic_vector (31 downto 0);
        SysAddr		:	in std_logic_vector (11 downto 0)
    );
end V4SmplFfsEmifC64x;

architecture V4SmplFfsEmifC64x_arch of V4SmplFfsEmifC64x is
----------------------------------------------------------------------------------------------------------------------------------------
-- Component Instantiation
----------------------------------------------------------------------------------------------------------------------------------------
component IBUF_LVCMOS33		port	(I : in std_logic; O : out std_logic); 								end component;
component IOBUF_LVCMOS33	port	( I, T : in std_logic; O : out std_logic; IO : inout std_logic); 	end component;
component BUFIO 			port	(I : in std_logic; O : out std_logic); 								end component;
component BUFR				generic (BUFR_DIVIDE : string);
							port	(I, CLR, CE : std_logic; O : out std_logic); 						end component;
component OBUF_LVCMOS33		port	( I: in std_logic; O: out std_logic); 								end component;
component IBUFG_LVCMOS33	port	(I : in std_logic; O : out std_logic); 								end component;

component RAMB16_S4_S4
	port (
		DOA          : out std_logic_vector (3 downto 0);
		DOB          : out std_logic_vector (3 downto 0);
		
		ADDRA        : in  std_logic_vector (11 downto 0);
		ADDRB        : in  std_logic_vector (11 downto 0);
		CLKA         : in  std_logic;
		CLKB         : in  std_logic;
		DIA          : in  std_logic_vector (3 downto 0);
		DIB          : in  std_logic_vector (3 downto 0);
		ENA          : in  std_logic;
		ENB          : in  std_logic;
		SSRA         : in  std_logic;
		SSRB         : in  std_logic;
		WEA          : in  std_logic;
		WEB          : in  std_logic
		);
end component;

component DCM_BASE
	generic (
	  CLKDV_DIVIDE 			: real;
	  CLKFX_DIVIDE 			: integer;
	  CLKFX_MULTIPLY 		: integer;
	  CLKIN_PERIOD 			: real;
	  CLK_FEEDBACK 			: string;
	  DESKEW_ADJUST 		: string;
	  DFS_FREQUENCY_MODE 	: string;
	  DLL_FREQUENCY_MODE 	: string;
	  DUTY_CYCLE_CORRECTION : boolean
	  );
	port (
	  CLK0 		: out std_logic;
	  CLK180 	: out std_logic;
	  CLK270 	: out std_logic;
	  CLK2X 	: out std_logic;
	  CLK2X180 	: out std_logic;
	  CLK90 	: out std_logic;
	  CLKDV 	: out std_logic;
	  CLKFX 	: out std_logic;
	  CLKFX180 	: out std_logic;
	  LOCKED 	: out std_logic;
	  CLKFB 	: in std_logic;
	  CLKIN 	: in std_logic;
	  RST 		: in std_logic
	  );
end component;

component IBUFG port (I : in std_logic; O : out std_logic); end component;
component BUFG	port (I : in std_logic; O : out std_logic); end component;
component FDP	port (C	: in std_logic;	D : in std_logic; PRE : in std_logic;	Q : out std_logic);	end component;

----------------------------------------------------------------------------------------------------------------------------------------
-- Signal, Constants and Attributes Declarations
----------------------------------------------------------------------------------------------------------------------------------------
-- Constants

-- Signals
signal High : std_logic := '1';
signal Low	: std_logic := '0';
------------------ EMIF
signal IntClkToIoBuf 	: std_logic;
signal IntIoBufToBufr	: std_logic;
signal IntClk			: std_logic;
signal IntClkDiv		: std_logic;
signal IntClkDivNot		: std_logic;

signal IntDatIn			: std_logic_vector (31 downto 0);
signal IntDatInBytEn	: std_logic_vector (31 downto 0);
signal IntDatOut 		: std_logic_vector (31 downto 0);
signal IntBytEn 		: std_logic_vector (3 downto 0);
signal IntBytEnNot 		: std_logic_vector (3 downto 0);
signal IntAddr 			: std_logic_vector (11 downto 0);
signal IntAddrFfs		: std_logic_vector (11 downto 0);

signal IntCe 		: std_logic;
signal IntWr 		: std_logic;
signal IntOe 		: std_logic;
signal IntCeNot 	: std_logic;
signal IntWrNot 	: std_logic;
signal IntOeNot 	: std_logic;
signal IntCeFf 		: std_logic;
signal IntWrFf 		: std_logic;

------------------ System
signal IntRst 		: std_logic;
signal IntRstNot 	: std_logic;
signal IntSysClkIbufgToDcm	: std_logic;
signal IntSysClk0ToBufg 	: std_logic;
signal IntSysClk			: std_logic;
signal IntSysClkLocked		: std_logic;
signal IntSysClkLockedNot 	: std_logic;
signal IntSysRst		: std_logic;
signal IntSysEna 		: std_logic;
signal IntSysEnaFf1 	: std_logic;
signal IntSysEnaFf2 	: std_logic;
signal IntSysWre 		: std_logic;
signal IntSysWreFf1 	: std_logic;
signal IntSysWreFf2 	: std_logic;
signal IntSysWreNot		: std_logic;
signal IntSysAddr 		: std_logic_vector(11 downto 0);
signal IntSysAddrFf1 	: std_logic_vector(11 downto 0);
signal IntSysAddrFf2 	: std_logic_vector(11 downto 0);
signal IntSysDatIn 		: std_logic_vector(31 downto 0);
signal IntSysDatInFf1 	: std_logic_vector(31 downto 0);
signal IntSysDatInFf2 	: std_logic_vector(31 downto 0);
signal IntSysDatOut 	: std_logic_vector(31 downto 0);
signal IntSysDatOutFf1 	: std_logic_vector(31 downto 0);
signal IntSysDatOutFf2 	: std_logic_vector(31 downto 0);
-- Attributes
attribute IOB : string;
--	attribute IOB of IntSysAddrFf1 : label is "FALSE";
----------------------------------------------------------------------------------------------------------------------------------------
--
begin

-- EMIF ECLKOUTx Clock
-- In this design the RAMB16 components are clocked on the negative edge of IntClkDiv.
-- While the Input FFs are clocked on the rising edge of IntClk.
-- EclkOut input PAD to input-FFs clock routing delay is: ~2.430ns (Tiopi+0.338+0.000+1.136)
-- EclkOut input PAD to RAMB16,CLK clock routing delay is ~2.560ns (Tiopi+0.338+0.000+0.101+0,212+0.990)

TheEmifClk :	IBUF_LVCMOS33	port map (I => DspEclkOut,		O => IntClkToIoBuf);
ClkIoBuf :	BUFIO			port map (I => IntClkToIoBuf,	O => IntIoBufToBufr);
ClkBufr :	BUFR			generic map (BUFR_DIVIDE => "BYPASS")
											-- BUFFER_DIVIDE => "BYPASS" results in a BUFR delay of 0.212 ns
											-- BUFFER_DIVIDE => "1" results in a BUFR delay of 0.732 ns
							port map (I => IntIoBufToBufr,	CE => Low,	CLR => Low,	O => IntClkDiv);
IntClk <= IntIoBufToBufr;
IntClkDivNot <= not IntClkDiv;

-- EMIF Address, Byte_Enable and data (in and out)
-- Byte_Enable is active low.
-- Data input to the blockRAM gets anded with ByteEnable.
-- Data output is a straight output.
TheEmifAddr : for n in 11 downto 0 generate
	AddrBuf : IBUF_LVCMOS33 port map (I => DspEA(n), O => IntAddr(n));
	DspAddrFfs : process (IntClk, IntRstNot)
	begin
		if (IntRstNot = '1') then
			IntAddrFfs(n) <= '0';
		elsif (IntClk'event and IntClk = '1') then
			IntAddrFfs(n) <= IntAddr(n);
		end if;
	end process;
end generate;
--
TheEmifBytEn : for n in 3 downto 0 generate
	EnBuf : IBUF_LVCMOS33 port map (I => DspBEn(n), O => IntBytEn(n));
end generate;
--
TheEmifDat : for n in 31 downto 0 generate
	IoBuf : IOBUF_LVCMOS33 port map (I => IntDatOut(n), O => IntDatIn(n), T => IntOeNot, IO => DspED(n));
	Zer: if n <= 7 generate
		DspZerFfs : process (IntClk, IntRstNot)
		begin
			if (IntRstNot = '1') then
				IntDatInBytEn(n) <= '0';
			elsif (IntClk'event and IntClk = '1') then
				if (IntBytEn(0) = '0') then
					IntDatInBytEn(n) <= IntDatIn(n);
				end if;
			end if;
		end process;
	end generate;
	One : if n >= 8 and n <= 15 generate
		DspOneFfs : process (IntClk, IntRstNot)
		begin
			if (IntRstNot = '1') then
				IntDatInBytEn(n) <= '0';
			elsif (IntClk'event and IntClk = '1') then
				if (IntBytEn(1) = '0') then
					IntDatInBytEn(n) <= IntDatIn(n);
				end if;
			end if;
		end process;
	end generate;
	Two : if n >= 16 and n <= 23 generate
		DspTwoFfs : process (IntClk, IntRstNot)
		begin
			if (IntRstNot = '1') then
				IntDatInBytEn(n) <= '0';
			elsif (IntClk'event and IntClk = '1') then
				if (IntBytEn(2) = '0') then
					IntDatInBytEn(n) <= IntDatIn(n);
				end if;
			end if;
		end process;
	end generate;
	Tre : if n >= 24 and n <= 31 generate
		DspTreFfs : process (IntClk, IntRstNot)
		begin
			if (IntRstNot = '1') then
				IntDatInBytEn(n) <= '0';
			elsif (IntClk'event and IntClk = '1') then
				if (IntBytEn(3) = '0') then
					IntDatInBytEn(n) <= IntDatIn(n);
				end if;
			end if;
		end process;
	end generate;
end generate;

-- EMIF Control.
-- All signals are active low.
CeBuf	:	IBUF_LVCMOS33	port map (I => DspCEn, O => IntCe);
OeBuf	:	IBUF_LVCMOS33	port map (I => DspSOEn, O => IntOe);	-- Enable the output tri-state buffers.
IntOeNot <= not IntOe;
WrBuf	:	IBUF_LVCMOS33	port map (I => DspSWEn, O => IntWr);

DspCtrlFfs : process (IntClk, IntRstNot)
begin
	if (IntRstNot = '1') then
		IntCeFf <= '0';
		IntWrFf <= '0';
	elsif (IntClk'event and IntClk = '1') then 
		IntCeFf <= IntCe;
		IntWrFf <= IntWr;
	end if;
end process;
IntCeNot <= not IntCeFf;
IntWrNot <= not IntWrFf;

-- Block SelectRAM
-- Port A is EMIF side.
-- Port B is system side.
GenRam : for n in 0 to 7 generate
	DspRam : RAMB16_S4_S4
	port map (
		DOA          => IntDatOut ((n*4)+3 downto n*4),
		DOB          => IntSysDatOut ((n*4)+3 downto n*4),
	
		ADDRA        => IntAddrFfs,
		ADDRB        => IntSysAddr,
		CLKA         => IntClkDivNot,
		CLKB         => IntSysClk,
		DIA          => IntDatInBytEn ((n*4)+3 downto n*4),
		DIB          => IntSysDatIn ((n*4)+3 downto n*4),
		ENA          => IntCeNot,
		ENB          => IntSysEna,
		SSRA         => IntRstNot,
		SSRB         => IntSysRst,
		WEA          => IntWrNot,
		WEB          => IntSysWre
	);
end generate;
-----------------------------------------------------------------------------------------------------
-- System design
-----------------------------------------------------------------------------------------------------
-- Kept very simple in this case, car this is normally the place where custom
-- logic is connected.
--
------------------------------------------------------
-- Reset, Resest is active low in this case.
------------------------------------------------------
RstBuf	:	IBUF_LVCMOS33	port map (I => Rst, O => IntRst);
IntRstNot <= not IntRst;

------------------------------------------------------
-- System Clock input and DCM
------------------------------------------------------
TheSysClk : IBUFG port map (I => FpgaClk, O => IntSysClkIbufgToDcm);

TheSysDcm : DCM_BASE
	generic map (
	  CLKDV_DIVIDE 			=> 2.0,
	  CLKFX_DIVIDE 			=> 1,
	  CLKFX_MULTIPLY 		=> 4,                                
	  CLKIN_PERIOD 			=> 5.5,  -- 180 MHz
	  CLK_FEEDBACK 			=> "1X",
	  DESKEW_ADJUST 		=> "SYSTEM_SYNCHRONOUS",
	  DFS_FREQUENCY_MODE 	=> "LOW",
	  DLL_FREQUENCY_MODE 	=> "LOW",
	  DUTY_CYCLE_CORRECTION => TRUE
	  )
	port map (
	  CLK0 		=> IntSysClk0ToBufg,
	  CLK180 	=> open,
	  CLK270 	=> open,
	  CLK2X 	=> open,
	  CLK2X180 	=> open,
	  CLK90 	=> open,
	  CLKDV 	=> open,
	  CLKFX 	=> open,
	  CLKFX180 	=> open,
	  LOCKED 	=> IntSysClkLocked,
	  CLKFB 	=> IntSysClk,
	  CLKIN 	=> IntSysClkIbufgToDcm,
	  RST 		=> IntRstNot
	  );
TheSysBufg : BUFG	port map (I => IntSysClk0ToBufg, O => IntSysClk);
--
-- generate a registered reset signal for all FPGA logic.
IntSysClkLockedNot <= not IntSysClkLocked ;

SysRstFf : FDP					
	port map (
		D => IntSysClkLockedNot,
		C => IntSysClk,
		PRE => IntRstNot,
		Q => IntSysRst		-- Fpga system reset.
		);
------------------------------------------------------
-- System logic
-- This back end design is just for sake of demo.
------------------------------------------------------
-- Output
-- BRAM DOB ---IntSysDataOut--- FF ---IntSysDataOutFf1--- FF --IntSysDataOutFf2-- OBUF --SysDatIo
-- BRAM DIB ---IntSysDataIn---- FF ---IntSysDataInFf1---- FF --IntSysDataInFf2--- IBUF --SysDatIo
-- BRAM ADDRB -IntSysAddr------ FF ---IntSysAddrFf1------ FF --IntSysAddrFf2----- IBUF --SysAddr
-- BRAM ENB ---IntSysEna------- FF ---IntSysEnaFf1------- FF --IntSysEnaFf2------ IBUF --SysEna
-- BRAM WEB ---IntSysWre------- FF ---IntSysWreFf1------- FF --IntSysWreFf2------ IBUF --SysWre
process (IntSysClk, IntSysRst)
begin
	if (IntSysRst = '1') then
		IntSysDatOutFf1 <= (others => '0');
		IntSysDatOutFf2 <= (others => '0');

		IntSysDatInFf1  <= (others => '0');
		IntSysDatIn  <= (others => '0');
		
		IntSysEnaFf1     <= '0';
		IntSysEna     <= '0';
		
		IntSysWreFf1     <= '0';
		IntSysWre     <= '0';
		
		IntSysAddrFf1    <= (others => '0');
		IntSysAddr    <= (others => '0');
	elsif (IntSysClk'event and IntSysClk = '1') then
		IntSysDatOutFf2 <= IntSysDatOutFf1;
		IntSysDatOutFf1 <= IntSysDatOut;
		
		IntSysDatInFf1 <= IntSysDatInFf2;
		IntSysDatIn <= IntSysDatInFf1;
		
		IntSysAddrFf1 <= IntSysAddrFf2;
		IntSysAddr <= IntSysAddrFf1;
		
		IntSysEnaFf1 <= IntSysEnaFf2;
		IntSysEna <= IntSysEnaFf1;
		
		IntSysWreFf1 <= IntSysWreFf2;
		IntSysWre <= IntSysWreFf1;
	end if;
end process;

TheSysDat : for n in 31 downto 0 generate
	SysIoBuf : IOBUF_LVCMOS33 port map (I => IntSysDatOutFf2(n), O => IntSysDatInFf2(n), T => IntSysWreNot, IO => SysDatIo(n));
end generate;
TheSysAddr : for n in 11 downto 0 generate
	SysAddrBuf : IBUF_LVCMOS33 port map (I => SysAddr(n), O => IntSysAddrFf2(n));
end generate;

SysEnaBuf	:	IBUF_LVCMOS33	port map (I => SysEna, O => IntSysEnaFf2);
SysWrBuf	:	IBUF_LVCMOS33	port map (I => SysWre, O => IntSysWreFf2);
IntSysWreNot <= not IntSysWreFf2;

--
end V4SmplFfsEmifC64x_arch ;